#include <Audio.h>
#include <Wire.h>
#include <SPI.h>
#include <SD.h>
#include <SerialFlash.h>
#include <Bounce.h>
#include <LiquidCrystal_PCF8574.h>

// GUItool: begin automatically generated code
AudioPlaySdWav           sdwav;     //xy=377.2000274658203,240.1999969482422
AudioOutputI2S           speaker;           //xy=549.2000274658203,240.20001220703125
AudioConnection          patchCord1(sdwav, 0, speaker, 0);
AudioConnection          patchCord2(sdwav, 1, speaker, 1);
AudioControlSGTL5000     sgtl5000;     //xy=551.2000274658203,278.20005798339844
// GUItool: end automatically generated code

const int playBtn = 0;
const int prevBtn = 1;
const int nextBtn = 2;
const int ledPin = 13;

const int FILES_LIMIT = 10;

Bounce bouncePlay = Bounce(playBtn, 50);
Bounce bouncePrev = Bounce(prevBtn, 50);
Bounce bounceNext = Bounce(nextBtn, 50);

LiquidCrystal_PCF8574 lcd(0x27);

String audioFiles[FILES_LIMIT];
int filesCount = 0, currentFile = 0;
bool isPaused = true;

void setup() {
  pinMode(ledPin, OUTPUT);

  pinMode(playBtn, INPUT_PULLUP);
  pinMode(prevBtn, INPUT_PULLUP);
  pinMode(nextBtn, INPUT_PULLUP);
  
  lcd.begin(16, 2);
  lcd.setBacklight(true);
  lcd.clear();
  
  Serial.begin(9600);

  SPI.setMOSI(7);
  SPI.setSCK(14);
  if(!SD.begin(10)) {
    displayError("SD init. failed!");
  }

  getAudioFiles(audioFiles, FILES_LIMIT, &filesCount);
  if(filesCount == 0) {
    displayError("No audio files!");
  }

  AudioMemory(15);
  sgtl5000.enable();
  sgtl5000.volume(0.6);
}

void loop() {
  if(!isPaused && !sdwav.isPlaying()) {
    currentFile++;
    currentFile %= filesCount;
    playSound(currentFile);
  }

  if(sdwav.isPlaying()) {
    int total_s = sdwav.lengthMillis() / 1000;
    int elapsed_s = sdwav.positionMillis() / 1000;
    int total_m = total_s / 60;
    int elapsed_m = elapsed_s / 60;

    total_s -= total_m * 60;
    elapsed_s -= elapsed_m * 60;

    lcd.setCursor(0,1);
    if(elapsed_m < 10) lcd.print("0");
    lcd.print(elapsed_m);
    lcd.print(":");
    if(elapsed_s < 10) lcd.print("0");
    lcd.print(elapsed_s);
    lcd.print(" / ");
    if(total_m < 10) lcd.print("0");
    lcd.print(total_m);
    lcd.print(":");
    if(total_s < 10) lcd.print("0");
    lcd.print(total_s);
  }
  
  if(!bouncePlay.update() && !bouncePrev.update() && !bounceNext.update()) return;
  
  if(bouncePlay.fallingEdge()) {
    if(sdwav.isPlaying()) {
      lcd.clear();
      sdwav.stop();
      isPaused = true;
    } else {
      playSound(currentFile);
    }
  } else if(bouncePrev.fallingEdge()) {
    currentFile--;
    if(currentFile < 0) currentFile = filesCount - 1;
    playSound(currentFile);
  } else if(bounceNext.fallingEdge()) {
    currentFile++;
    currentFile %= filesCount;
    playSound(currentFile);
  }
}

void playSound(int index) {
  char file[40];
  audioFiles[index].toCharArray(file, 40);
  sdwav.play(file);
  lcd.clear();
  lcd.print(audioFiles[index]);
  isPaused = false;
  currentFile = index;
}

void getAudioFiles(String * fileList, int arraySize, int * filesCount) {
  File root = SD.open("/");
  *filesCount = 0;
  while(true) {
    File entry = root.openNextFile();
    if(!entry) break;
  
    String fileName = entry.name();
    fileName.toUpperCase();
    if(fileName.endsWith(".WAV")) {
      fileList[(*filesCount)++] = entry.name();
    }
    
    entry.close();
    if(*filesCount >= arraySize) break;
  }
}

void displayError(String error) {
  lcd.clear();
  lcd.setCursor(0,0);
  lcd.print("ERROR:");
  lcd.setCursor(0,1);
  lcd.print(error);
  while(true) {
    digitalWrite(ledPin, !digitalRead(ledPin));
    delay(500);
  }
}
